# -*- coding: utf-8 -*-
__version_info__ = '0.4.0'
__version__ = '0.4.0'
version = '0.4.0'
def get_version(): return version  # pragma: no cover

default_app_config = 'stagesetting.apps.StageSettingAppConfig'
