{% extends "stagesetting/base.html" %}
{% load i18n %}
{% block content %}
<p>{% blocktrans with escaped_object=original.key %}Are you sure you want to reset {{ escaped_object }} to it's default value?{% endblocktrans %}</p>

<form action="{% url 'stagesetting_delete' original.pk %}" method="post" accept-charset="utf-8">
        {% csrf_token %}
        {{ form.as_ul }}
        <input type="submit" value="{% trans "Yes, I'm sure" %}">
    </form>
{% endblock %}
